// Demo client for ID3gWS SOAP API
// Author: Phil Ashby
// Version: 1.2-CXF

import java.io.FileInputStream;
import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.xml.ws.BindingProvider;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.ws.security.SecurityConstants;

import com.id3global.ID3Global;
import com.id3global.IGlobalCredentials;
import com.id3global.IGlobalDocImage;
import com.id3global.IGlobalProfile;
import com.id3global.IGlobalSearch;
import com.id3global.id3gws._2013._04.ArrayOfGlobalProfileVersion;
import com.id3global.id3gws._2013._04.GlobalAccount;
import com.id3global.id3gws._2013._04.GlobalDIVData;
import com.id3global.id3gws._2013._04.GlobalDIVDataGlobalDocumentExtractedStatus;
import com.id3global.id3gws._2013._04.GlobalPEPIntelligenceData;
import com.id3global.id3gws._2013._04.GlobalProfileVersion;

public class ID3demoCXF {

	public static void main(String[] args) {
		new ID3demoCXF().go(args);
	}

	private void go(String[] args) {
		String user = null;
		String pass = null;
		String url = "https://pilot.id3global.com/ID3gWS/ID3global.svc?wsdl";
		String img = null;
		boolean fiddle = false;
		boolean nocred = false;
		boolean search = false;
		for (int arg=0; arg<args.length; arg++) {
			if (args[arg].startsWith("-u"))
				user = args[++arg];
			else if (args[arg].startsWith("-p"))
				pass = args[++arg];
			else if (args[arg].startsWith("-f"))
				fiddle = true;
			else if (args[arg].startsWith("-w"))
				url = args[++arg];
			else if (args[arg].startsWith("-n"))
				nocred = true;
			else if (args[arg].startsWith("-i"))
				img = args[++arg];
			else if (args[arg].startsWith("-s"))
				search = true;
		}
		if (null==user || null==pass) {
			System.err.println("usage: <ID3demoCXF -u <user> -p <password> [-n] [-f] [-i <image file>] [-w <WSDL url>]");
			System.err.println("       where: -n bypasses credential check, -f enables use of Fiddler 2 to monitor traffic.");
			System.err.println("              -i <image file> makes an image verification check");
			return;
		}
		if (fiddle) {
			try {
				// Kludge SSL certificate checking so we can use Fiddler2 and set proxy appropriately
				SSLContext sc = SSLContext.getInstance("SSL");
				TrustAllCerts trustAll = new TrustAllCerts();
				sc.init(null, new TrustManager[] { trustAll }, new java.security.SecureRandom());
				HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
				HttpsURLConnection.setDefaultHostnameVerifier(trustAll);
				System.setProperty("https.proxyHost", "localhost");
				System.setProperty("https.proxyPort", "8888");
			} catch (Exception e) {
				e.printStackTrace();
				return;
			}
		}

		// Sample ID3global session, checks credentials and reads out profiles
		System.out.println("Connecting(fiddler="+fiddle+"): " + url);
		ID3Global client = null;
		try {
			String orgID = "made up org";
			client = new ID3Global(new URL(url));
			if (!nocred) {
				// Always use the getWsHttpBindingXX() methods where available to permit WS-SecureConversation authentication
				IGlobalCredentials creds = client.getWsHttpBindingGlobalCredentials();
				suppressValidation(creds);
				System.out.println("Checking credentials");
				GlobalAccount ourAcc = creds.checkCredentials(user, pass);
				System.out.println("Got account: "+ourAcc.getAccountID()+" = "+ourAcc.getName()+"@"+ourAcc.getDomainName()+" / "+ourAcc.getOrgID());
				orgID = ourAcc.getOrgID();
			}
			if (!search) {
				IGlobalProfile profs = client.getWsHttpBindingGlobalProfile();
				// Associate supplied username/password with client proxy class, separated into a re-usable method
				addCredentials((BindingProvider)profs, user, pass);
				suppressValidation(profs);
				System.out.println("Getting profiles");
				ArrayOfGlobalProfileVersion ourProfs = profs.getProfileVersions(orgID);
				System.out.println("Profiles:");
				for (GlobalProfileVersion prof: ourProfs.getGlobalProfileVersion()) {
					System.out.println("\t" + prof.getName() + "/" + prof.getVersion());
				}
			} else {
				IGlobalSearch srch = client.getBasicHttpBindingGlobalSearch();
				addCredentials((BindingProvider)srch, user, pass);
				suppressValidation(srch);
				System.out.println("Getting PEP Data");
				GlobalPEPIntelligenceData intel = srch.getPEPIntelligenceData(orgID, "300156571");	// magic number from their bug report
				System.out.println("Got fullname: " + intel.getFullname());
			}
			if (img != null) {
				FileInputStream fin = new FileInputStream(img);
				byte[] imgdata = new byte[fin.available()];
				fin.read(imgdata);
				fin.close();
				IGlobalDocImage docimg = client.getBasicHttpBindingGlobalDocImage();		// No WsHttp version? oddness.
				addCredentials((BindingProvider)docimg, user, pass);
				suppressValidation(docimg);
				System.out.println("Uploading image");
				GlobalDIVData divData = docimg.uploadAndProcess(imgdata);
				if (divData != null && divData.getStatus() != null && divData.getAuthenticationID() != null) {
					String fore = "";
					String surn = "";
					if (divData.getExtractedData() != null) {
						fore = divData.getExtractedData().getForename();
						surn = divData.getExtractedData().getSurname();
					}
					System.err.println("result=" + divData.getStatus().value() +
						" authenticationId=" + divData.getAuthenticationID() +
						" forename=" + fore + " surname=" + surn);
					while (GlobalDIVDataGlobalDocumentExtractedStatus.IN_PROGRESS.equals(divData.getStatus())) {
						Thread.sleep(1000);
						System.out.println("checking for completion..");
						divData = docimg.getImageResults(orgID, divData.getAuthenticationID());
					}
				} else {
					System.err.println("no data returned");
				}
			}
			client = null;
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (client != null)
				System.err.println("oops");
		}
	}

	// Common method to associate credentials with a proxy class
 	private void addCredentials(BindingProvider bp, String user, String pass) {		
 		if (user!=null && pass!=null) {
 			// Standard JAX-WS properties
 			bp.getRequestContext().put(BindingProvider.USERNAME_PROPERTY, user);
 			bp.getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, pass);
 			// CXF proprietary properties, **plus very badly documented modification for older versions of CXF (pre 3.0) to support WS-SecureConversation - sigh**
			// http://cxf.apache.org/docs/ws-secureconversation.html - see example - this is the ONLY place it's mentioned.
			// https://docs.jboss.org/author/display/JBWS/WS-Security - JBoss documentation that mentions the trailing ".sct"
			// https://issues.apache.org/jira/browse/CXF-4457 - Bug reported to Apache, and fix to use unmodified property names in CXF 3.0+
 			bp.getRequestContext().put(SecurityConstants.USERNAME + ".sct", user);
 			bp.getRequestContext().put(SecurityConstants.PASSWORD + ".sct", pass);
			// CXF proprietary properties for FXF 3.0 onwards - still ought to use standard names..
			bp.getRequestContext().put(SecurityConstants.USERNAME, user);
 			bp.getRequestContext().put(SecurityConstants.PASSWORD, pass);
 		}
 	}

 	private void suppressValidation(Object o) {
 		// CXF-specific property set on the Endpoint to suppress validation 
 		Client client = ClientProxy.getClient(o);
 		client.getEndpoint().put("set-jaxb-validation-event-handler", "false");
 	}
	// Special TrustManager and HostnameVerifier implementation that removes all TLS/SSL security..
	// please do NOT use in production code. This is here to permit Fiddler 2 to intercept the SSL
	// traffic for debugging purposes ONLY.
	private class TrustAllCerts implements X509TrustManager, HostnameVerifier
	{
		public void checkClientTrusted(X509Certificate[] chain, String authType)
				throws CertificateException {
		}

		public void checkServerTrusted(X509Certificate[] chain, String authType)
				throws CertificateException {
		}

		public X509Certificate[] getAcceptedIssuers() {
			// TODO Auto-generated method stub
			return new X509Certificate[0];
		}

		public boolean verify(String arg0, SSLSession arg1) {
			// TODO Auto-generated method stub
			return true;
		}		
	};
}
